function [Z,Q,SSdist] = zprocess(p)

%[Z, Zpers, Ztrans, Q] = yprocess(p) is the program to compute the income
%process. 
%
%Inputs: 
%   p           Structure of parameters
%
%Output:
%   Z           Income states
%   Q           Transition matrix
%   SSdist      Distribution of states
%
%Author: Swapnil Singh
%Date: August 26, 2014

%0. Preliminaries
%----------------
eval(['load ' 'var_covar_cex_1980_2003.out'])
eval(['yautocovvardata =  var_covar_cex_1980_2003;' ]);

%******
%Note that the data contains moments from 1980_ to 2003.
%Right now I will only keep the 2003 moments but we can change the
%code any time to include other year data moments.
%
%The first column is the variance, and the second column the covariance
%
%********
yautocovvardata(1:end-6, :) = [];     %Keep only the year 2003 moments

%1. Variance of the persistent and transitory part of the income
% ---------------------------------------------------------------
if p.rho ~= 0

Varpers = mean(yautocovvardata(1:5,2))/p.rho ;   %variance of the persistent part of the income process 
%Varpers = 0.2496/p.rho;                                                %is same as the variance of
                                                 %income as we are condidering
                                                 %simple income process


Vartrans = mean(yautocovvardata(1:5,1)) - Varpers;
%Vartrans = 0.3578 - Varpers;

%Variance of the transitory part of the income
else
    Varpers = mean(yautocovvardata(1:5,1));
    Vartrans = 0;
end
fprintf('Variance of Income from data = %f\n', mean(yautocovvardata(1:5,1)));
%fprintf('Variance of Income from data = %f\n', 0.3578);

%2. Create the income process (focus is on Tauchen Hussey method of
%discretization)
%------------------------------------------------------------------

if p.tauchen == 1 %discretize using the Tauchen(1986) procedure
   if p.ntrans == 1   %if there is no two state iid transitory shocks
        Q=ss_tauchen(p.npers,p.rho,p.m);            %this tauchen code is different from Floden's in the point that we provide our own m
        [SSdist,~]=eigs(Q',1);
        SSdist=SSdist/sum(SSdist);                  % stationary distribution
        ns=length(SSdist);                          % number of states
        grid=2*(0:ns-1)/(ns-1)-1;                   %equally distributed points
        varly=mean(yautocovvardata(1:5,1));         %variance of log income from the data
        lyh=sqrt(varly/((grid.^2)*SSdist));         %calibration of income distribution
        Z=exp(lyh*grid);
        
   else   %if there are transitory shocks
        Q=ss_tauchen(p.npers,p.rho,p.m);            %this tauchen code is different from Floden's in the point that we provide our own m
        [pip,~]=eigs(Q',1);
        pip=pip/sum(pip);                           % stationary distribution over persistent states
        nsp=length(pip);                            % number of persistent states
        grid=2*(0:p.npers-1)/(p.npers-1)-1;         % equally distributed points
        lyhz=sqrt(Varpers/((grid.^2)*pip)); 
        lyhe=sqrt(Vartrans);
        [lya,orda]=sort([lyhz*grid-repmat(lyhe,1,nsp) lyhz*grid+repmat(lyhe,1,nsp)],2); 
        Z=exp(lya);                                 %Income states for all years and states
        pi=[pip/2; pip/2]';%Invariant distribution income
        ord=orda(1,:);  %Here we can pick a year, first year 1980 is equal to 1
        SSdist = pi(ord);
        y=Z(1,:);%Here we can pick a year, first year 1980 is equal to 1
        ybar=pi*y';
        Pt2=[[Q/2; Q/2] [Q/2; Q/2]];
        Q=Pt2(ord,ord);


   end
elseif p.tauchen == 0  %Use the Tauchen Hussey procedure for the discretization of the shock
    
    if p.ntrans == 1   %if there is no two state iid transitory shocks
        [~,Q]=tauchenhussey(p.npers,0, p.rho,1,1); % transition probabilities
        [SSdist,~]=eigs(Q',1);
        SSdist=SSdist/sum(SSdist);                  % stationary distribution
        ns=length(SSdist);                          % number of states
        grid=2*(0:ns-1)/(ns-1)-1;                   %equally distributed points
        varly=mean(yautocovvardata(1:5,1));         %variance of log income from the data
        lyh=sqrt(varly/((grid.^2)*SSdist));         %calibration of income distribution
        Z=exp(lyh*grid);
        
   else   %if there are transitory shocks
        [~,Qp]=tauchenhussey(p.npers,0, p.rho,1,1);  % transition probabilities
        [pip,~]=eigs(Qp',1);
        pip=pip/sum(pip);                           % stationary distribution over persistent states
        nsp=length(pip);                            % number of persistent states
        grid=2*(0:p.npers-1)/(p.npers-1)-1;         % equally distributed points
        lyhz=sqrt(Varpers/((grid.^2)*pip));
        lyhe=sqrt(Vartrans);
        [lya,orda]=sort([lyhz*grid-repmat(lyhe,1,nsp) lyhz*grid+repmat(lyhe,1,nsp)],2); 
        Z=exp(lya);  %Income states for all years and states
        pi=[pip/2; pip/2]';%Invariant distribution income
        ord=orda(1,:);  %Here we can pick a year, first year 1980 is equal to 1
        SSdist = pi(ord);
        y=Z(1,:);%Here we can pick a year, first year 1980 is equal to 1
        ybar=pi*y';
        Pt2=[[Qp/2; Qp/2] [Qp/2; Qp/2]];
        Q=Pt2(ord,ord);
        




   end
    
end



fprintf('Variance of income process created = %f\n', var(log(Z(1,:)),SSdist));


% 3. Sort the income and distribution accordingly
% -----------------------------------------------
[Zlog, index] = sort(Z);
Qtemp = zeros(size(Q));
QQtemp = Qtemp;
SSdisttemp = zeros(p.S,1);
for j = 1 : p.S
   Qtemp(j,:) = Q(index(j),:);
   SSdisttemp(j) = SSdist(index(j));
end
for j = 1 : p.S
   QQtemp(:,j) = Qtemp(:,index(j)); 
end
Q = QQtemp;
SSdist = SSdisttemp;

%4. Final Manipulations
%----------------------
if size(Z,1) > size(Z,2)
   Z = Z';      %Z is a row vector now 
end
if size(SSdist, 1)< size(SSdist,2)
   SSdist = SSdist';            %SSdist is a column vector now 
end
Z = Z/(SSdist'* Z');

%5. Save the SSdist for the conditional moments calculation from the data
%------------------------------------------------------------------------
cd 'Results'
save SSdist.mat SSdist
save Zprocess.mat Z
cd ..

end