function pers_aiyagari_ni()

%pers_aiyagari_ni() solves the Aiyagari no information model with capital.
%
%Inputs: None
%
%Outputs: None
%
%Author: Swapnil Singh and Christian Stoltenberg
%
%Version: 
%  February 3, 2019
%

%1. Load the data
%----------------
load('parameters_pers_aiyagari_ni.mat');

    %1.2 Declare the save file name
   
    savefilename = 'pers_aiyagari_ni';
    
    
    fprintf('Number of persistent income states = %i\n', p.npers);
    fprintf('Number of transitory income states = %i\n', p.ntrans);
    fprintf('Persistent parameter = %f\n', p.rho );
    fprintf('Interest rate = %f\n', p.R);
    fprintf('Number of grid points = %i\n', p.nk);
    fprintf('Results are saved in filename = %s \n', savefilename)


    
%2. Grid for income
%------------------
[Z,Q,SSdist] = zprocess(p);

%3. Compute the optimal discount factor
%--------------------------------------
    
    %3.1 Search interval
    %-------------------
    b1 = 0.9; b2 = 1/p.R;
    
    %3.2 Apply the Ridder's Algorithm
    %--------------------------------
    d1 = solve_model(b1, p,kgrid, Z, Q,SSdist, 0,savefilename); %difference between aggregate supply and demand of capital at r1
    d2 = solve_model(b2, p,kgrid, Z, Q,SSdist, 0,savefilename); %difference between aggregate supply and demand of capital at r2

    %check whether root lies in between r1 and r2
    if d1*d2 > 0
        error('Root do not lies in between the given values of interest rates');
    end
    
    if d1 == 0 || d2 == 0
       error('You are already at the solution point');
    end
    
    bl = b1; bh = b2;
    dl = d1; dh = d2;
    conv = 1e-6;
    iter = 0;
    
    while true
       bm = 0.5*(bl + bh);
       dm = solve_model(bm, p,kgrid, Z, Q,SSdist, 0,savefilename);
       if sign(dm^2 - dl*dh) > 0
          sq = sqrt(dm^2 - dl*dh);
       else
           error('There is something wrong. Please check');
       end
       if sq == 0
          error('Dinominator is zero'); 
       end
       
       bnew = bm + (bm - bl)*sign(dl-dh)*dm/sq;
       if abs(bnew - conv) < 1e-6
         fprintf('Converged subjective discount factor is = %f\n', bnew)
         dnew = solve_model(bnew, p,kgrid, Z, Q,SSdist, 1,savefilename);
         p.beta = bnew;
         cd 'Aiyagari/Results'
         save(savefilename,'p', '-append');
         cd ..
         cd ..
         break;
       end
       fprintf('Difference between old and new beta: %f\n',(abs(bnew - conv)));
       conv = bnew;
       dnew = solve_model(conv, p,kgrid, Z, Q,SSdist, 0,savefilename);
       
       if abs(dm)*sign(dnew) ~= dm
          bl = bm;
          dl = dm;
          bh = conv;
          dh = dnew;
       elseif abs(dl)*sign(dnew) ~= dl
           bh = conv;
           dh = dnew;
       elseif abs(dh)*sign(dnew) ~= dh
           bl = conv;
           dl = dnew;
       else
           error('There is something wrong with the code')
       end
       iter = iter + 1;
    end

   disp('--------------------------------------------------------------------');
   fprintf('Checking beta*R < 1\n')
   load(savefilename)
   if (p.beta* p.R < 1)
      fprintf('Checked: R*beta <1\n') 
   else
       error(' R*beta >=1. Convergence criteria not satisfied\n') 
   end
   
   
end

%-------------------------------------------------------------------------%
%----------- Excess Demand Function --------------------------------------%
%-------------------------------------------------------------------------%
function ed = solve_model(b, p,kgrid, Z, Q,SSdist, i,savefilename)

% ed = solve_model(b, p,kgrid, Z, Q, i,savefilename) solves for the excess
% demand function for the given discount factor.
%
%Inputs: 
%       b               Given discount factor
%       p               Structure of parameters
%       kgrid           Grid of capital
%       Z               income states
%       pinf            transition matrix
%       SSdist          Stationary distribution of income states
%       i               indicator to save the results
%       savefilename    File to save all the results
%
%Output: 
%       ed              Excess demand
%
%Author: Swapnil Singh and Christian Stoltenberg
%

p.beta = b;     %assign the given subjective discount factor
R = p.R;        %gross interest rate
r = R-1;        %Net Interest rate
k_agg = ((r + p.delta)/(p.AA*p.alpha))^(1/(p.alpha - 1));       %aggregate demand of capital
wage = (1 - p.alpha) *p.AA* k_agg^p.alpha;                      %wage rate

%transform the income
Z = wage * Z;


%1. Value function iteration
%---------------------------
[~,d] = vfi_aiyagari(p, kgrid, Z,Q,r,wage);

%2. Next period capital
%----------------------
kp = kgrid(d);
kp= kp(:);
 
%3. Transition matrix
%--------------------
QQ = trans_aiyagari(d, Q);
[f, ~]=eigs(QQ',1);     %stationary distribution
f = f/sum(f);           %normalize the distribution
f(f<p.pcrit) = 0;       %kill the numerical error

%4. Market Clearing Condition
%----------------------------
as = kp'*f;
ed = as - k_agg;


    f(f<1e-16) = 0;                      %kill the computational error probability measures
    f = reshape(f, p.nk, p.S);          %reshape f in the original matrix form
    l = ones(p.nk, p.S);                %create the matrix for the shock process
    for i = 1 : p.S 
       l(:,i) = Z(i); 
    end

c = l + (1 + r)*repmat(kgrid,1,p.S) - reshape(kp, p.nk,p.S);      %consumption for Aiyagari no information model
index = c < 0;                                                      %check whether consumption is negative or not
if sum(sum(index)) > 0
   error('Consumption is negative');
end
ed2 = (c(:)'*f(:) + p.delta *kp(:)'*f(:) - p.AA * k_agg^p.alpha)/k_agg;

fprintf('---------------------------------------------------\n');
fprintf('Supply - demand capital = %f \n', ed);
fprintf('Goods market clearing = %f \n', ed2)
fprintf('---------------------------------------------------\n');



%also compute the insurance ratio
 lc = log(c);
 %compute the unconditional variance of consumption
    vc_ani = f(:)'*lc(:).^2 - (f(:)'*lc(:))^2;
    vZ_ani = SSdist'*log(Z').^2 - (SSdist'*log(Z'))^2;

    vcvZ = vc_ani/vZ_ani;
    IR_invar = 1 - vcvZ

%5. Compute and save the results
%-------------------------------
% if i == 1
    
    cd 'Aiyagari/Results'
        save(savefilename);
    cd ..
    cd ..
    
% end



end

function [v,d] =  vfi_aiyagari(p, kgrid, y, pinf,r,w)


%vfi_aiyagari(p, kgrid, y, pinf,r,w) is the value function iteration
%
%
%Inputs: 
%       p       Structure of parameters
%       kgrid   Grid of capital
%       y       Income states
%       pinf    Transition matrix
%       r       Given interest rate
%       w       Given wage rate
%
%Outputs:
%       v       value function
%       d       Policy function
%



    %1.algorithm parameters
    %----------------------
    crit = 1;                       %convergence criteria
    epsi = 1e-5;                    %convergence parameter

    %2. Storage matrices
    %-------------------
    v = zeros(p.nk, p.S) ;         %value function


    %Function Handle for utility
    %---------------------------
    if p.sigma == 1
       u = @(c) log(c);
    else
        u = @(c) (c.^(1 - p.sigma) - 1)/(1 - p.sigma);
    end


    %3. Operational matrices
    %-----------------------
    o = ones(p.nk,1);
    oo = ones(p.nk);
    R = cell(p.S,1);

    k1 = repmat(kgrid, 1, p.nk);

    k2 = repmat(kgrid', p.nk,1);
    A = [];
    for j = 1 : p.S
       R{j} = (1 + r)*k1 +  w * oo * y(j) - k2;
       R{j}(R{j}<=0) = NaN;
       R{j} = u(R{j});
       A = [A;R{j}];
    end

    while crit > epsi
       temp = (A + p.beta * kron(pinf, o) * v');
       [tv,d] = max(temp, [], 2);
       tv = reshape(tv,p.nk,p.S );
       d = reshape(d,p.nk,p.S);
       crit = max(max(abs(tv - v)));
       v = tv;
    end

end   %end of the value function iteration

function QQ = trans_aiyagari(d, PI)



%another way to create the transition matrix
[n,m] = size(d);
d = d(:);
nn = length(d);

QQ = zeros(nn,nn);

for i1 = 1 : nn
   
    %find which income state are we in
    if i1 < nn
        y = floor(i1/n) + 1;
    else
        y = m;
    end
        
    %create the index
    index = (0:m-1)*n + d(i1);
    
    QQ(i1, index) = PI(y, :);
    
    
end


end   %end of the trans_aiyagari function


